/**
 * 请假逻辑
 * 作者:lxm
 * 包括工时填报
 */

import moment = require("moment");
import { APPROVETYPE, ASKFORLEAVETYPE, STATED } from "../config/enum";
// import { addData, findAllAskforLeave } from "../data/admin/askForLeave";
import * as tools from "../util/tools";
import { addAskforleave, delAskForLeaveListById, findAskForLeaveCountByTime, findAskForLeaveListByApprover, findAskForLeaveListById, findAskForLeaveListByTime, findAskForLeaveListByTimeToPage, findAskForLeaveListByUserId, findAskforleaveByUserIdAndType, updateAskForLeaveState, updateAskforleave } from "../data/admin/askForLeave";
import { findDepartmentRole, findUserByUserIdNum } from "../data/admin/user";
import { BizError } from "../util/bizError";
import { ERRORENUM } from "../config/errorEnum";
import { findOvertimeApprove } from "../data/admin/overtime";
import * as userData from "../data/admin/user";


/**
 * 添加请假记录
 * @param userId 填报人
 * @param desc 描述
 * @param startTime 开始请假时间
 * @param endTime 结束请假时间
 * @param type 请假类型
 * @param date 填报时间
 * @returns {isSuccess:true, message:"添加成功"}
 */
export async function addAskForLeaveLog(userId:number, des:string, startTime:string, endTime:string, type:number, day:number, time:number) {
    const DateKey = tools.getDateKey();
    let st = moment(startTime).format("YYYY-MM-DD HH:mm:ss");
    let et = moment(endTime).format("YYYY-MM-DD HH:mm:ss");

    let id = tools.getAskForLeaveId(userId);

    let userInfo = await findUserByUserIdNum(userId);
    let approver = 0;
    if (userInfo.roleId == 6) {
        let approverInfo = await findDepartmentRole(5, userInfo.departmentId);
        approver = approverInfo.userId;
    } else {
        approver = 16;//默认给万仪婷审批
    }

    if (type == ASKFORLEAVETYPE.年假) {
        let timeCount = (day*8) + time;
        if (timeCount > userInfo.annualLeave) throw new BizError(ERRORENUM.年假剩余不足请重新分配请假时间);
    }

    if (type == ASKFORLEAVETYPE.调休) {
        let askforleaveCount = await findAskforleaveByUserIdAndType(userId, ASKFORLEAVETYPE.调休);
        let approveCount = await findOvertimeApprove(userId);
        let residue = approveCount - askforleaveCount;
        let timeCount = (day*8) + time;
        if (timeCount > residue) throw new BizError(ERRORENUM.调休剩余不足请重新分配请假时间)
    }

    await addAskforleave(id, des, st, et, userId, approver, time, day, type, DateKey);

    return {isSuccess:true, message:"添加成功"};
}


export async function myAskForLeaveList(userId:number) {
    let dataList = await findAskForLeaveListByUserId(userId);

    let notApproveList = [];
    let notPassingList = [];
    let passingList = [];
    dataList.forEach(info => {
        let { id, des, time, type, day, state, notPassing, dateKey } = info;
        let changeType = tools.changeEnumValue(ASKFORLEAVETYPE, type);
        let startTime = moment(info.startTime).format("YYYY/MM/DD HH:mm");
        let endTime = moment(info.endTime).format("YYYY/MM/DD HH:mm");
        let timeStr = `${day}日${time}小时`;
        if (new Date(dateKey).valueOf() > ( new Date().valueOf() - (3600*1000*24*80)) ) {
            //未超过80天
            if (state == STATED.否) notApproveList.push({id, des, startTime, endTime, timeStr, changeType, type, day, time});
            else if (state == APPROVETYPE.通过) passingList.push({des, startTime, endTime, timeStr, changeType});
            else if (state == APPROVETYPE.不通过) notPassingList.push({des, startTime, endTime, timeStr, changeType, notPassing});
        } else {
            //超过80天
            if (state == STATED.否) notApproveList.push({id, des, startTime, endTime, timeStr, changeType, type, day, time});
        }
    });

    let askforleaveCount = await findAskforleaveByUserIdAndType(userId, ASKFORLEAVETYPE.调休);
    let approveCount = await findOvertimeApprove(userId);
    let residue = approveCount - askforleaveCount;

    let userInfo = await findUserByUserIdNum(userId);

    let annualLeave = userInfo.annualLeave || 0;

    return {
        notApproveList, 
        notPassingList, 
        passingList, 
        residueDay:Math.floor(residue/8),
        residueTime: residue%8,
        annualLeaveDay : Math.floor(annualLeave/8),
        annualLeaveTime : annualLeave%8,
    };
}


export async function deleteAskForLeave(id:string ) {
    let askForLeaveInfo = await findAskForLeaveListById(id);

    if (askForLeaveInfo.state == STATED.是) throw new BizError(ERRORENUM.已经通过审批的请假不能修改或删除);

    await delAskForLeaveListById(id);

    return {isSuccess:true, message:"删除成功"};
}


export async function updateAskForLeaveLog(id:string, userId:number, des:string, startTime:string, endTime:string, type:number, day:number, time:number ) {
    let overTimeInfo = await findAskForLeaveListById(id);

    if (overTimeInfo.state == STATED.是) throw new BizError(ERRORENUM.已经通过审批的加班不能修改或删除);

    await updateAskforleave(id, des, startTime, endTime, time, day, type);

    return {isSuccess:true, message:"添加成功"};
}



export async function approveAskForLeave(id:string, type:number, notPassingStr:string) {
    if (type == APPROVETYPE.不通过 && !notPassingStr) {
        throw new BizError(ERRORENUM.参数错误);
    }
    if (type == APPROVETYPE.通过) notPassingStr = "";

    let askForLeaveList = await findAskForLeaveListById(id);
    let askForLeaveInfo = askForLeaveList[0];
    if (!askForLeaveInfo) throw new BizError(ERRORENUM.目标数据已不存在);
    
    if (askForLeaveInfo.type == ASKFORLEAVETYPE.年假) {
        let userInfo = await findUserByUserIdNum(askForLeaveInfo.userId);
        let annualLeave = userInfo.annualLeave - ((askForLeaveInfo.day * 8 ) + askForLeaveInfo.time);
        await userData.updateData({annualLeave:annualLeave}, {userId:userInfo.userId});
    }
    await updateAskForLeaveState(id, type, notPassingStr);

    return {isSuccess:true, message:"操作成功"};
}

export async function approveAskForLeaveList(userId:number) {

    let askForLeaveList = await findAskForLeaveListByApprover(userId);
    let dataList = [];
    let peopleMap = {};
    askForLeaveList.forEach(info => {
        let {userName, startTime, endTime, day, time, type, des, id} = info;
        dataList.push({
            startTime:moment(startTime).format("YYYY-MM-DD HH:mm"),
            endTime:moment(endTime).format("YYYY-MM-DD HH:mm"),
            timeStr : `${day}日${time}小时`,
            type:tools.changeEnumValue(ASKFORLEAVETYPE, type), 
            des,
            userName,
            id
        });
        peopleMap[userName] = 1;
    });

    return {dataList, peopleCount:Object.keys(peopleMap).length };
}


export async function getPersonnelAFLList(page:number, userId:number, startTime:string, endTime:string) {

    let selectParam:any = {};
    if (userId) selectParam.userId = userId;
    if (startTime) selectParam.startTime = moment( startTime).format("YYYY-MM-DD")+" 00:00:00";
    if (endTime) selectParam.endTime = moment( endTime).format("YYYY-MM-DD")+" 00:00:00";

    let dbList = await findAskForLeaveListByTimeToPage(selectParam, (page-1)*10, 10);

    let count = await findAskForLeaveCountByTime(selectParam);

    let dataList = [];
    dbList.forEach(info => {
        let {name, startTime, endTime, day, time, type, des, state, approval } = info;

        let approvalStr = "";
        if (approval == 16) approvalStr = "万仪婷";
        else if (approval == 2) approvalStr = "李欣鸣";
        else approvalStr = "蔺荣儿";

        let stateStr = "";
        if (state == 0) stateStr = "未审批";
        else if (state == 1) stateStr = "已通过";
        else stateStr = "未通过";
        dataList.push({
            name, 
            startTime:moment( startTime).format("YYYY-MM-DD HH:mm:SS"), 
            endTime:moment( endTime).format("YYYY-MM-DD HH:mm:SS"), 
            long:(day*8)+ time,
            type:tools.changeEnumValue(ASKFORLEAVETYPE, type), des, 
            state:stateStr, 
            approval:approvalStr
        });
    });

    return {dataList, count};
}


export async function getMemberNameList() {
    let result = await userData.selectMemberUser();
    let dataList = [];
    result.forEach(info => {
        dataList.push({
            value:info.userId,
            label:info.name
        });
    });
    return {dataList};
}


export async function getOutPutPersonnelAFLList(userId:number, startTime:string, endTime:string) {
    let selectParam:any = {};
    if (userId) selectParam.userId = userId;
    if (startTime) selectParam.startTime = moment( startTime).format("YYYY-MM-DD")+" 00:00:00";
    if (endTime) selectParam.endTime = moment( endTime).format("YYYY-MM-DD")+" 00:00:00";

    let dbList = await findAskForLeaveListByTime(selectParam);
    let dataList = [];
    dbList.forEach(info => {
        let {name, startTime, endTime, day, time, type, des, state, approval } = info;

        let approvalStr = "";
        if (approval == 16) approvalStr = "万仪婷";
        else if (approval == 2) approvalStr = "李欣鸣";
        else approvalStr = "蔺荣儿";

        let stateStr = "";
        if (state == 0) stateStr = "未审批";
        else if (state == 1) stateStr = "已通过";
        else stateStr = "未通过";
        dataList.push({
            name, 
            startTime:moment( startTime).format("YYYY-MM-DD HH:mm:SS"), 
            endTime:moment( endTime).format("YYYY-MM-DD HH:mm:SS"), 
            long:(day*8)+ time,
            type:tools.changeEnumValue(ASKFORLEAVETYPE, type), des, 
            state:stateStr, 
            approval:approvalStr
        });
    });
    return {dataList};
}