/**
 * 统计主要逻辑
 */

import moment = require("moment");
import { DEPARTMENT } from "../config/enum";
import { findAllProject, findAllProjectMap, findProjectByName } from "../data/admin/project";
import { getThisMonthWorkingLogByProject, getThisMonthWorkingLogByProjectAndUserName, statsAllProjectTime, statsProjectTimeByTime, statsThisMonthProjectWorkingLog, statsThisMonthWorkingLog, statsTimeByDepartment } from "../data/admin/workinglog";
import { selectUserByDepartmentId } from "../data/admin/user";
import { findOvertimeByDepartmentId } from "../data/admin/overtime";
import { findAskForLeaveByDepartmentId } from "../data/admin/askForLeave";

export async function allProjectData() {
    
    let projectMap = await findAllProjectMap();

    let dataList = await statsAllProjectTime();

    let map = {};
    dataList.forEach(info => {
        let {time, projectId, departmentId} = info;
        if (!map[projectId]) map[projectId] = {};
        if (!map[projectId][departmentId]) map[projectId][departmentId] = time;
    });

    let projectList = [];
    for (let projectId in map) {
        let info = map[projectId];
        let ui = parseInt(info[DEPARTMENT.设计] || 0);
        let dev = parseInt(info[DEPARTMENT.开发] || 0);
        let pm = parseInt(info[DEPARTMENT.项目经理] || 0);
        projectList.push({
            ui, dev, pm, 
            count:ui+dev+pm,
            projectName:projectMap[projectId],
        });
    }
    projectList.sort((a,b) => {return b.count - a.count});

    let ui = [];
    let dev = [];
    let pm = [];
    let projectNameList = [];
    let pieChartList = [];
    const PieChartDataCount = 8;
    projectList.forEach((info, index) => {
        ui.push(info.ui);
        dev.push(info.dev);
        pm.push(info.pm);
        projectNameList.push(info.projectName);
        
        if (index < PieChartDataCount) {
            pieChartList.push({name:info.projectName, value:info.count});
        } else {
            if (!pieChartList[PieChartDataCount]) pieChartList.push({name:"其他", value:info.count});
            else pieChartList[PieChartDataCount].value += info.count;
        }
    });

    

    return { ui, dev, pm, projectNameList, pieChartList };
}

export async function timeProjectData(st:string, et:string) {
    
    let projectMap = await findAllProjectMap();


    let dataList = await statsProjectTimeByTime(st, et);

    let map = {};
    dataList.forEach(info => {
        let {time, projectId, departmentId} = info;
        if (!map[projectId]) map[projectId] = {};
        if (!map[projectId][departmentId]) map[projectId][departmentId] = time;
    });

    let projectList = [];
    for (let projectId in map) {
        let info = map[projectId];
        let ui = parseInt(info[DEPARTMENT.设计] || 0);
        let dev = parseInt(info[DEPARTMENT.开发] || 0);
        let pm = parseInt(info[DEPARTMENT.项目经理] || 0);
        projectList.push({
            ui, dev, pm, 
            count:ui+dev+pm,
            projectName:projectMap[projectId],
        });
    }
    projectList.sort((a,b) => {return b.count - a.count});

    let ui = [];
    let dev = [];
    let pm = [];
    let projectNameList = [];
    let pieChartList = [];
    const PieChartDataCount = 8;
    projectList.forEach((info, index) => {
        ui.push(info.ui);
        dev.push(info.dev);
        pm.push(info.pm);
        projectNameList.push(info.projectName);
        
        if (index < PieChartDataCount) {
            pieChartList.push({name:info.projectName, value:info.count});
        } else {
            if (!pieChartList[PieChartDataCount]) pieChartList.push({name:"其他", value:info.count});
            else pieChartList[PieChartDataCount].value += info.count;
        }
    });

    

    return { ui, dev, pm, projectNameList, pieChartList };
}

export async function departmentWorkinglogData(departmentId:number) {
    let thisDepartmentUserList = await selectUserByDepartmentId(departmentId);

    let startTime = moment().format("YYYY-MM")+'-01 00:00:00';
    let worklogMap = await statsTimeByDepartment(departmentId, startTime);
    let nowMs = new Date(moment().format("YYYY-MM-DD")+' 00:00:00').valueOf();
    /**算应填的时间 */
    let workDayCount = 0;
    for (let i = 0; i < moment().daysInMonth(); i++) {
        let ms = moment(startTime).add(i, 'days').valueOf();
        let weekDay = moment(ms).day();
        if (weekDay <=5 && weekDay !=0) {
            workDayCount += 8;
        }
        if (ms == nowMs) break;
    }

    let completedList = []
    let notCompletedList = [];
    let nameList = [];
    thisDepartmentUserList.forEach(info => {
        let {name} = info;
        let time = worklogMap[name] || 0;
        
        completedList.push(time);
        notCompletedList.push(workDayCount - time);
        nameList.push(name);
    });

    return {completedList, notCompletedList, nameList};

}


export async function departmentAskForLeaveAndOverTimeData(departmentId:number) {
    let thisDepartmentUserList = await selectUserByDepartmentId(departmentId);

    let selectTime = moment().format("YYYY-MM")+'-01 00:00:00';
    let overtimeDataMap = await findOvertimeByDepartmentId(departmentId, selectTime);
    let askForLeaveMap = await findAskForLeaveByDepartmentId(departmentId, selectTime);

    let overTimeList = []
    let askForLeaveList = [];
    let nameList = [];

    thisDepartmentUserList.forEach(info => {
        let {name} = info;
        let overTime = overtimeDataMap[name] || 0;
        let askForLeaveTime = askForLeaveMap[name] || 0;
        
        overTimeList.push(overTime);
        askForLeaveList.push(askForLeaveTime);
        nameList.push(name);
    });

    return { overTimeList, askForLeaveList, nameList };
}

export async function departmentProjectData(departmentId:number) {
    let selectTime = moment().format("YYYY-MM")+'-01 00:00:00';

    let dbList = await statsThisMonthWorkingLog(departmentId, selectTime);

    let dataMap = {};
    dbList.forEach(info => {
        let {projectName, time, userName} = info;
        if (!dataMap[projectName]) dataMap[projectName] = {};
        dataMap[projectName][userName] = parseInt(time);
    });

    let dataList = [];
    for (let name in dataMap) {
        let addInfo = {name, children:[]};
        for (let userName in dataMap[name]) {
            let value = dataMap[name][userName];
            let key = `${userName}(${value})`;
            addInfo.children.push({
                name:key,
                value
            });
        }
        dataList.push(addInfo);
    }

    return { dataList };
}

export async function departmentProjectCalendarData(departmentId:number) {
    let selectTime = moment().format("YYYY-MM")+'-01 00:00:00';

    let workLogList = await statsThisMonthProjectWorkingLog(departmentId, selectTime);
    let nowMs = new Date(moment().format("YYYY-MM-DD")+' 00:00:00').valueOf();
    let dataMap = {};
    let distinctMap = {};
    workLogList.forEach(info => {
        let {projectName, time, dateKey} = info;
        distinctMap[projectName] = 1;
        let key = moment(dateKey).format("YYYY-MM-DD");
        if (!dataMap[key]) dataMap[key] = {};

        if (!dataMap[key][projectName]) dataMap[key][projectName] = 0;
        dataMap[key][projectName] += parseInt(time);
    });

    let dataList = [];
    let scatterData = [];
    for (let i = 0; i < moment().daysInMonth(); i++) {
        let ms = moment(selectTime).add(i, 'days').valueOf();
        let dateKey = moment(ms).format("YYYY-MM-DD");
        if (ms < nowMs) {
            let onceData = dataMap[dateKey];
            let data = [];
            for (let projectName in onceData) {
                data.push({name:projectName, value:onceData[projectName]});
            }
            dataList.push({center:dateKey, data });
        } else {
            dataList.push({center:dateKey, data:[] });
        }
        scatterData.push([dateKey]);
    }
    let month = moment().format("YYYY-MM");

    return {dataList, projectList:Object.keys(distinctMap), month, scatterData}
}

export async function getThisMonthProject(projectName:string, departmentId:number) {
    let projectList = await findProjectByName(projectName);
    if (!projectList.length) return {dataList:[]}
    let projectId = projectList[0].id;
    let selectTime = moment().format("YYYY-MM")+'-01 00:00:00';

    let dataBaseList = await getThisMonthWorkingLogByProject(departmentId, selectTime, projectId);
    let dataList = [];
    dataBaseList.forEach(info => {
        let {name, dateKey, workLog, time} = info; 
        dataList.push({name, date:moment(dateKey).format("YYYY-MM-DD"), workLog, time});
    });
    return {dataList}
}


export async function getThisMonthProjectAndName(projectName:string, departmentId:number, userName:string) {
    let projectList = await findProjectByName(projectName);
    if (!projectList.length) return {dataList:[]}
    let projectId = projectList[0].id;
    let selectTime = moment().format("YYYY-MM")+'-01 00:00:00';

    let dataBaseList = await getThisMonthWorkingLogByProjectAndUserName(departmentId, selectTime, projectId, userName);
    let dataList = [];
    dataBaseList.forEach(info => {
        let {name, dateKey, workLog, time} = info; 
        dataList.push({name, date:moment(dateKey).format("YYYY-MM-DD"), workLog, time});
    });
    return {dataList}
}