/**
 * 用户管理
 * 作者:lxm
 * 包括 登录 改密码 获取权限密码
 */

import { DEPARTMENT, ROLE } from "../config/enum";
import { ERRORENUM } from "../config/errorEnum";
import { FIRSTLVLIMIT, SECONDLVLIMIT } from "../config/systemConfig";
import * as userData from "../data/admin/user";
import { BizError } from "../util/bizError";
import { getToken } from "../util/tools";


export async function login(loginId:string, pwd:number) {
    let userInfo = await userData.findUserByLoginId(loginId);
    if (!userInfo) throw new BizError(ERRORENUM.登录失败, '用户不存在');

    if (userInfo.pwd != pwd) throw new BizError(ERRORENUM.密码错误, `密码错误 ${pwd}`);

    let token = getToken(userInfo.loginId);

    await userData.updateData({token}, {loginId});

    let stagingList = [];
    let approveList = [];
    let statsList = [];//统计
    if (userInfo.departmentId == DEPARTMENT.开发 || userInfo.departmentId == DEPARTMENT.设计 ||userInfo.departmentId == DEPARTMENT.项目经理) {
        stagingList = [FIRSTLVLIMIT.工时填报, FIRSTLVLIMIT.加班填报, FIRSTLVLIMIT.请假填报];
        if (userInfo.roleId == ROLE.部门经理) {
            stagingList.push(FIRSTLVLIMIT.数据导出);
            stagingList.push(FIRSTLVLIMIT.部门工时列表)
            statsList = [SECONDLVLIMIT.工时统计, SECONDLVLIMIT.部门统计];
            if (userInfo.departmentId != DEPARTMENT.项目经理) approveList = [FIRSTLVLIMIT.加班审批, FIRSTLVLIMIT.请假审批];
        }
    } else if (userInfo.departmentId == DEPARTMENT.领导) {
        stagingList = [FIRSTLVLIMIT.请假审批, FIRSTLVLIMIT.数据导出];
        statsList = [SECONDLVLIMIT.工时统计, SECONDLVLIMIT.部门统计];
    } else if (userInfo.departmentId == DEPARTMENT.行政 ) {
        stagingList = [FIRSTLVLIMIT.加班审批, FIRSTLVLIMIT.数据导出, FIRSTLVLIMIT.立项];
    } else if (userInfo.departmentId == DEPARTMENT.人事) {
        stagingList = [FIRSTLVLIMIT.人事用请假情况, FIRSTLVLIMIT.请假审批, FIRSTLVLIMIT.出勤];
    }

    let result = { token, 
        userId:userInfo.userId, 
        roleId:userInfo.roleId, 
        departmentId:userInfo.departmentId, 
        name:userInfo.name,
        headSculpture:`http://192.168.0.105:7088/1.png`,
        stagingList,
        approveList,
        statsList
    };

    return result;
}

export async function logout(userId:number) {
    let userInfo = await userData.findUserByUserIdNum(userId);
    if ( !userInfo ) throw new BizError(ERRORENUM.身份验证失败);
    await userData.updateData({token:""}, {userId});
    return {isSuccess:true}
}

export async function changePwd() {
    
}
