/**请假 */

import { APPROVETYPE } from "../../config/enum";
import { mysqlDB } from "../../db/mysqlInit";
const Sequelize =require('sequelize')
//创建StudentModel模型，该模型对应的表名是student

let askforleaveModel;
export async function initModel() {
    var model = mysqlDB.define('askforleave',{
        id:{
            type:Sequelize.STRING(100), //表示属性的数据类型
            primaryKey:true,  //表示主键
            allowNull:false,   //表示当前列是否允许为空，false表示该列不能为空
            unique:true    //表示该列的值必须唯一
        },
        des:{ type:Sequelize.STRING(100), allowNull:false }, //事由
        startTime:{ type:Sequelize.DATE, allowNull:false },//开始时间
        endTime:{ type:Sequelize.DATE, allowNull:false },//结束时间
        userId:{ type:Sequelize.INTEGER, allowNull: false },//填报人
        approval:{ type:Sequelize.INTEGER, allowNull: false },//审批人
        time:{ type:Sequelize.INTEGER, allowNull: false},//请假时长 小时
        day:{ type:Sequelize.INTEGER, allowNull: false},//请假时长 天
        type:{ type:Sequelize.INTEGER, allowNull: false},//类型
        dateKey:{ type:Sequelize.DATE, allowNull:false },//填报日期
        state:{ type:Sequelize.INTEGER, allowNull: false},//状态
        notPassing:{ type:Sequelize.STRING(100), allowNull:false }, //不通过原因
    },{
        freezeTableName:true, //true表示使用给定的表名，false表示模型名后加s作为表名
        timestamps:false  //true表示给模型加上时间戳属性(createAt、updateAt),false表示不带时间戳属性
    });
    // //同步数据库，force的值为false，表若存在则先删除后创建，force的值为true表示表若存在则不创建
    askforleaveModel = await model.sync({});
}


export async function initAskforleave(id:string, des:string, startTime:string, endTime:string, userId:number, approval:number, time:number, day:number, type:number, dateKey:string, state:number) {
    await askforleaveModel.create({id, des, startTime, endTime, userId, approval, time, day, type, dateKey, notPassing:"", state});
}

/**
 * 添加请假
 */
export async function addAskforleave(id:string, des:string, startTime:string, endTime:string, userId:number, approval:number, time:number, day:number, type:number, dateKey:string) {
    await askforleaveModel.create({id, des, startTime, endTime, userId, approval, time, day, type, dateKey, notPassing:"", state:0});
}


export async function findAskforleaveByUserIdAndType(userId:number, type:number) {
    let list = await askforleaveModel.findAll({
        where:{ userId, type, state:APPROVETYPE.通过 }
    });
    let count = 0;
    list.forEach(info => {
        let {day, time} = info;
        if (day!=0) count += day*8;
        if (time != 0) count += time;
    });

    return count;
}

export async function findAskForLeaveListByUserId(userId:number) {
    return await askforleaveModel.findAll({where:{userId}});
}

export async function findAskForLeaveListById(id:string) {
    return await askforleaveModel.findAll({where:{id}});
}

export async function delAskForLeaveListById(id:string) {
    await askforleaveModel.destroy({where:{id}});
}

export async function findAskForLeaveListByTimeToPage(param, startNum:number, endNum:number) {
    let sql = `select users.name, askforleave.state, askforleave.startTime, askforleave.endTime, askforleave.day, askforleave.time, askforleave.type,askforleave.des, askforleave.approval  from askforleave INNER JOIN users on users.userId = askforleave.userId `;

    let paramList = Object.keys(param)
    if (paramList.length) {
        sql += ' where ';
        for (let i= 0; i < paramList.length; i++) {
            let key = paramList[i];
            if (key == "startTime") {
                sql += `askforleave.startTime > '${param.startTime}' `;
            }
            if (key == "endTime") {
                sql += `askforleave.endTime < '${param.endTime}' `;
            }
            if (key == "userId") {
                sql += `users.userId = ${param.userId}`;
            }

            if (i != paramList.length -1) {
                sql += ' and '
            }
        }
       
    } 
    sql += ` ORDER BY askforleave.dateKey DESC limit ${startNum}, ${endNum}`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function findAskForLeaveListByTime(param) {
    let sql = `select users.name, askforleave.state, askforleave.startTime, askforleave.endTime, askforleave.day, askforleave.time, askforleave.type,askforleave.des, askforleave.approval  from askforleave INNER JOIN users on users.userId = askforleave.userId `;

    let paramList = Object.keys(param)
    if (paramList.length) {
        sql += ' where ';
        for (let i= 0; i < paramList.length; i++) {
            let key = paramList[i];
            if (key == "startTime") {
                sql += `askforleave.startTime > '${param.startTime}' `;
            }
            if (key == "endTime") {
                sql += `askforleave.endTime < '${param.endTime}' `;
            }
            if (key == "userId") {
                sql += `users.userId = ${param.userId}`;
            }

            if (i != paramList.length -1) {
                sql += ' and '
            }
        }
       
    } 
    sql += ` ORDER BY askforleave.dateKey DESC`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function findAskForLeaveCountByTime(param) {
    let sql = `select count(*) as count from askforleave`;
    let paramList = Object.keys(param)
    if (paramList.length) {
        sql += ' where ';
        for (let i= 0; i < paramList.length; i++) {
            let key = paramList[i];
            if (key == "startTime") {
                sql += `startTime > '${param.startTime}' `;
            }
            if (key == "endTime") {
                sql += `endTime < '${param.endTime}' `;
            }
            if (key == "userId") {
                sql += `userId = ${param.userId}`;
            }

            if (i != paramList.length -1) {
                sql += ' and '
            }
        }
       
    } 
    let countData = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    return countData[0].count;
}

export async function updateAskforleave(id:string, des:string, startTime:string, endTime:string,  time:number, day:number, type:number) {
    let updateInfo = {des, startTime, endTime, time, day, type};
    await askforleaveModel.update(updateInfo, {where:{id}});
}

export async function findAskForLeaveListByApprover(userId:number) {
    let sql = `select users.name as userName, askforleave.id, askforleave.startTime, askforleave.endTime, askforleave.day, askforleave.time, askforleave.type, askforleave.des  from askforleave INNER JOIN users on `;
    sql += `askforleave.userId = users.userId  where askforleave.approval = ${userId}  and askforleave.state = 0`
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function updateAskForLeaveState(id:string, state:number, notPassing?) {
    let updateInfo:any = {state};
    if (notPassing) updateInfo.notPassing = notPassing;
    await askforleaveModel.update(updateInfo, {where:{id}});
}


export async function findAskForLeaveByDepartmentId(departmentId:number, selectTime:string) {
    let sql =  `select users.name, sum(askforleave.time) as time, sum(askforleave.day) as day from users LEFT JOIN askforleave on askforleave.userId = users.userId WHERE users.departmentId = ${departmentId} and (askforleave.startTime > "${selectTime}" or askforleave.endTime > "${selectTime}") GROUP BY users.name `
    let dataList = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    let map = {};
    dataList.forEach(info => {
        map[info.name] = parseInt(info.time) + (parseInt(info.day) * 8 );
    });
    return map;
}