/**
 * 加班表
 */

import { mysqlDB } from "../../db/mysqlInit";
const Sequelize =require('sequelize')
//创建StudentModel模型，该模型对应的表名是student

let overtimeModel;
export async function initModel() {
    var model = mysqlDB.define('overtime',{
        id:{
            type:Sequelize.STRING(100), //表示属性的数据类型
            primaryKey:true,  //表示主键
            allowNull:false,   //表示当前列是否允许为空，false表示该列不能为空
            unique:true    //表示该列的值必须唯一
        },
        projectId:{ type:Sequelize.STRING(100), allowNull:false }, //项目号
        userId:{ type:Sequelize.INTEGER, allowNull: false },//填报人
        time:{ type:Sequelize.INTEGER, allowNull:false },//时长
        dateKey:{ type:Sequelize.DATE, allowNull:false },//填报时间
        state:{ type:Sequelize.INTEGER },//状态 0=否 1=通过 2=不通过
        workLog:{ type:Sequelize.STRING(200)}, //工作内容
        approver:{ type:Sequelize.INTEGER, allowNull: false },//审批人
        notPassing:{type:Sequelize.STRING(200)},//不通过理由
    },{
        freezeTableName:true, //true表示使用给定的表名，false表示模型名后加s作为表名
        timestamps:false  //true表示给模型加上时间戳属性(createAt、updateAt),false表示不带时间戳属性
    });
    // //同步数据库，force的值为false，表若存在则先删除后创建，force的值为true表示表若存在则不创建
    overtimeModel = await model.sync({});
}

/**
 * 添加工时记录
 * @param id 属性的数据类型
 * @param projectId 项目号
 * @param userId 填报人
 * @param time 工时
 * @param dateKey 填报时间
 * @param isOutside 是否外出 0=否 1=是
 * @param workLog 工作内容
 */
export async function addOvertime(id:string, projectId:string, userId:number, time:number, dateKey:string, workLog:string, approver:number) {
    await overtimeModel.create({id, projectId, userId, time, dateKey, state:0, workLog, approver});
}


export async function findOvertimeById(id:string) {
    return await overtimeModel.findAll({
        where:{id}
    });
}


export async function findOvertime(userId:number) {
    return await overtimeModel.findAll({
        where:{userId}
    });
}

export async function findOvertimeApprove(userId:number) {
    let list = await overtimeModel.findAll({
        where:{userId, state:1}
    });
    let count = 0;
    list.forEach(info => {
        let {time} = info;
        count += time;
    });

    return count;
}

export async function updateOvertime(id:string, projectId:string, time:number, workLog:string) {
    await overtimeModel.update({projectId, time, workLog}, {where:{id}});
}

export async function delOvertime(id:string) {
    await overtimeModel.destroy({where:{id}});
}

export async function updateOvertimeState(id:string, state:number, notPassing?) {
    let updateInfo:any = {state};
    if (notPassing) updateInfo.notPassing = notPassing;
    await overtimeModel.update(updateInfo, {where:{id}});
}

export async function findOvertimeByApprover(userId:number) {
    let sql = `select users.name as userName, project.name as projectName, overtime.id, overtime.time, overtime.dateKey, overtime.workLog from overtime INNER JOIN users on `;
    sql += `overtime.userId = users.userId INNER JOIN project on project.id = overtime.projectId where overtime.approver = ${userId} and overtime.state = 0`
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function initOvertime(id:string, projectId:string, userId:number, time:number, dateKey:string, workLog:string, approver:number, state:number) {
    await overtimeModel.create({id, projectId, userId, time, dateKey, state, workLog, approver, notPassing:""});
}

export async function findOvertimeByDepartmentId(departmentId:number, startTime:string) {
    let sql =  `select users.name, sum(overtime.time) as time from users LEFT JOIN overtime on overtime.userId = users.userId WHERE users.departmentId = ${departmentId} and overtime.dateKey >"${startTime}" GROUP BY users.name`
    let dataList = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    let map = {};
    dataList.forEach(info => {
        map[info.name] = info.time;
    });
    return map;
}

export async function findOvertimeByTime(startTime:string, endTime:string) {
    let sql =  `select users.name, project.name as projectName, overtime.time, overtime.dateKey from users LEFT JOIN overtime on overtime.userId = users.userId inner join project on project.id = overtime.projectId  `
    
    if (startTime && endTime) {
        sql += ` where overtime.dateKey > "${startTime}" and overtime.dateKey < "${endTime}"`;
    }

    let dataList = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    return dataList;
}