/**
 * 项目表
 */

const Sequelize =require('sequelize')
const MySequesize = require('../../db/mysqlInit');  //导入创建的sequelize对象
//创建StudentModel模型，该模型对应的表名是student

let projectModel;
export async function initModel() {
    var model = MySequesize.mysqlDB.define('project',{
        id:{//项目号
            type:Sequelize.STRING(200), 
            primaryKey:true,  //表示主键
            allowNull:false,   //表示当前列是否允许为空，false表示该列不能为空
            unique:true    //表示该列的值必须唯一
        },
        name:{ type:Sequelize.STRING(100), allowNull:false }, 
        contractId:{ type:Sequelize.STRING(200),  },//合同号
        controller:{ type:Sequelize.STRING(200) }, //项目经理
        createTime:{ type:Sequelize.DATE},// 创建时间
    },{
        freezeTableName:true, //true表示使用给定的表名，false表示模型名后加s作为表名
        timestamps:false  //true表示给模型加上时间戳属性(createAt、updateAt),false表示不带时间戳属性
    });
    projectModel = await model.sync({});

}

/**
 * 查询所有项目
 * @returns 
 */
export async function findAllProjectMap() {
    let projectList = await projectModel.findAll({});

    let result = {};
    projectList.forEach(info => {
        result[info.id] = info.name;
    });

    return result;
}


/**
 * 根据项目id查询项目
 * @param projectId 
 * @returns 
 */
export async function findOnceProjectById(projectId:string) {
    let dataList = await projectModel.findAll({
        where:{id:projectId}
    });
    return dataList[0] || {};
}


/**
 * 查询所有项目的id和名称
 * @returns 
 */
export async function findAllProject() {
    let dataList = await projectModel.findAll({
        where:{}
    });
    return dataList;
}

/**
 * 查询项目名称
 * @returns 
 */
export async function findProjectByName(projectName:string) {
    let dataList = await projectModel.findAll({
        where:{name:projectName}
    });
    return dataList;
}



export async function addProject(id:string, name:string, controller:string, createTime:string) {
    await projectModel.create({id, name, controller, createTime });
}

export async function updateProject(id:string, name:string, controller:string) {
    await projectModel.update({name, controller }, {where:{id}});
}