/**
 * 工时表
 */

import { mysqlDB } from "../../db/mysqlInit";
const Sequelize =require('sequelize')
//创建StudentModel模型，该模型对应的表名是student

let workinglogModel;
export async function initModel() {
    var model = mysqlDB.define('workinglog',{
        id:{
            type:Sequelize.STRING(100), //表示属性的数据类型
            primaryKey:true,  //表示主键
            allowNull:false,   //表示当前列是否允许为空，false表示该列不能为空
            unique:true    //表示该列的值必须唯一
        },
        projectId:{ type:Sequelize.STRING(100), allowNull:false }, //项目号
        userId:{ type:Sequelize.INTEGER, allowNull: false },//填报人
        time:{ type:Sequelize.INTEGER, allowNull:false },//工时
        dateKey:{ type:Sequelize.DATE, allowNull:false },//填报时间
        isOutside:{ type:Sequelize.INTEGER },//是否外出 0=否 1=是
        workLog:{ type:Sequelize.STRING(200)}, //工作内容
    },{
        freezeTableName:true, //true表示使用给定的表名，false表示模型名后加s作为表名
        timestamps:false  //true表示给模型加上时间戳属性(createAt、updateAt),false表示不带时间戳属性
    });
    // //同步数据库，force的值为false，表若存在则先删除后创建，force的值为true表示表若存在则不创建
    workinglogModel = await model.sync({});
}


/**
 * 获取今日时长
 * @param userId 
 * @param dateKey 
 * @returns 
 */
export async function findWorkingLogLengthByDateKey(userId:number, dateKey:string) {
    
    let count = 0;
    let dataList = await workinglogModel.findAll({
        where:{userId:userId, dateKey:dateKey}
    });
    dataList.forEach(info => {
        count += info.time;
    });
    return count;
}


/**
 * 获取本周工时记录
 * @param userId 
 * @param startTime 
 * @param endTime 
 * @returns 
 */
export async function findThisWeekWorkingLog(userId:number, startTime:string, endTime:string) {
    let sql = `select workinglog.workLog, workinglog.id as id,workinglog.projectId as projectId, project.name as projectName, workinglog.time, workinglog.dateKey, `;
    sql += `workinglog.isOutside from workinglog INNER JOIN project on project.id = workinglog.projectId where userId = ${userId} and dateKey > "${startTime}" and dateKey < "${endTime}" `
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    let dataList = [];
    res.forEach(info => {
        dataList.push(info);
    });
    return dataList;
}


/**
 * 获取所有工时记录  按时间分
 * 带分页
 * @param startIndex 
 * @param startTime 
 * @param endTime 
 * @returns 
 */
export async function findAllWorkingLogToPage(startIndex:number, startTime:string, endTime:string) {
    let sql = `SELECT workinglog.workLog, workinglog.projectId AS projectId, project.NAME AS projectName, workinglog.time, workinglog.dateKey, workinglog.isOutside, users.name, users.departmentId `;
    sql += `FROM workinglog INNER JOIN project ON project.id = workinglog.projectId inner JOIN users on users.userId = workinglog.userId `;
    if (startTime && endTime) {
        sql += ` where workinglog.dateKey > "${startTime}" and workinglog.dateKey < "${endTime}"`;
    }
    sql += `LIMIT ${startIndex},10`;
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    let dataList = [];
    res.forEach(info => {
        dataList.push(info);
    });
    let countSql = `SELECT workinglog.workLog, workinglog.projectId AS projectId, project.NAME AS projectName, workinglog.time, workinglog.dateKey, workinglog.isOutside, users.name, users.departmentId `;
    countSql += `FROM workinglog INNER JOIN project ON project.id = workinglog.projectId inner JOIN users on users.userId = workinglog.userId`
    if (startTime && endTime) {
        countSql += ` where dateKey > "${startTime}" and dateKey < "${endTime}"`;
    }
    let countRes = await mysqlDB.query(countSql, { type: mysqlDB.QueryTypes.SELECT } );
    return {dataList, count:countRes.length};
}


/**
 * 获取所有工时记录  按时间分
 * 不带分页
 * @param startTime 
 * @param endTime 
 * @returns 
 */
export async function findAllWorkingLog( startTime:string, endTime:string) {
    let sql = `SELECT workinglog.workLog, workinglog.projectId AS projectId, project.NAME AS projectName, workinglog.time, workinglog.dateKey, workinglog.isOutside, users.name, users.departmentId `;
    sql += `FROM workinglog INNER JOIN project ON project.id = workinglog.projectId inner JOIN users on users.userId = workinglog.userId `
    if (startTime && endTime) {
        sql += ` where workinglog.dateKey > "${startTime}" and workinglog.dateKey < "${endTime}"`;
    }
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    return res;
}


/**
 * 获取所有工时记录 按项目分
 * @param startTime 开始时间
 * @param endTime 结束时间
 * @returns 
 */
export async function findAllWorkingLogToProject(startTime:string, endTime:string) {
    let sql = `SELECT workinglog.projectId AS projectId, project.NAME AS projectName, workinglog.time `;
    sql += `FROM workinglog INNER JOIN project ON project.id = workinglog.projectId `;
    if (startTime && endTime) {
        sql += ` where workinglog.dateKey > "${startTime}" and workinglog.dateKey < "${endTime}"`;
    }
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } )
    return res;
}


/**
 * 添加工时记录
 * @param id 属性的数据类型
 * @param projectId 项目号
 * @param userId 填报人
 * @param time 工时
 * @param dateKey 填报时间
 * @param isOutside 是否外出 0=否 1=是
 * @param workLog 工作内容
 */
export async function addWorkingLog(id:string, projectId:string, userId:number, time:number, dateKey:string, isOutside:number, workLog:string) {
    // let sql = `INSERT INTO workinglog ( id, projectId, userId, time, dateKey, isOutside, workLog) values `;
    // sql += `( "${id}", "${projectId}", "${userId}", "${time}", "${dateKey}", "${isOutside}", "${workLog}" )`;
    await workinglogModel.create({id, projectId, userId, time, dateKey, isOutside, workLog});
}


/**
 * 修改工时记录
 * @param id 属性的数据类型
 * @param projectId 项目号
 * @param time 工时
 * @param isOutside 是否外出 0=否 1=是
 * @param workLog 工作内容
 */
export async function updateWorkinglog(id:string, projectId:string, time:number, isOutside:number, workLog:string) {
    let sql = `UPDATE workinglog SET projectId="${projectId}", time="${time}", isOutside="${isOutside}", workLog="${workLog}" where id="${id}"`;
    await workinglogModel.update({projectId, time, isOutside, workLog}, {where:{id}});
}


/**
 * 根据id查询工时记录
 * @param id 属性的数据类型
 * @returns 
 */
export async function findWorkingLogById(id:string) {
    let dataList = await workinglogModel.findAll({
        where:{id}
    });
    return dataList[0] || {};
}


/**
 * 根据id删除工时记录
 * @param id 属性的数据类型
 * @returns 
 */
export async function deleteWorkingLogById(id:string) {
    return await workinglogModel.destroy({where:{id}});
}

export async function statsAllProjectTime() {
    let sql = `select workinglog.projectId, users.departmentId, sum(workinglog.time) as time  from users INNER JOIN workinglog on workinglog.userId = users.userId GROUP BY users.departmentId, workinglog.projectId`;
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } )
    return res;
}

export async function statsProjectTimeByTime(st:string, et:string) {
    let sql = `select workinglog.projectId, users.departmentId, sum(workinglog.time) as time  from users INNER JOIN workinglog on workinglog.userId = users.userId where workinglog.dateKey > '${st}' and workinglog.dateKey < '${et}' GROUP BY users.departmentId, workinglog.projectId `;
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } )
    return res;
}

export async function statsTimeByDepartment(departmentId:number, st:string) {
    let sql = `select users.name, sum(workinglog.time) as time from users INNER JOIN workinglog on workinglog.userId = users.userId WHERE users.departmentId = ${departmentId} and dateKey>"${st}" GROUP BY users.name`;
    let data = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    let map = {};
    data.forEach(info => {
        map[info.name] = info.time;
    });

    return map;
}

export async function statsThisMonthWorkingLog(departmentId:number, st:string) {
    let sql = `select project.name as projectName, users.name as userName, sum(workinglog.time) as time from users INNER JOIN workinglog on workinglog.userId = users.userId INNER JOIN project on project.id = workinglog.projectId where users.departmentId = ${departmentId} and workinglog.dateKey >"${st}" GROUP BY project.name, users.name`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function statsThisMonthProjectWorkingLog(departmentId:number, st:string) {
    let sql = `select project.name as projectName, workinglog.time, workinglog.dateKey from project LEFT JOIN workinglog on workinglog.projectId= project.id INNER JOIN users on users.userId = workinglog.userId where workinglog.dateKey > "${st}" and users.departmentId = ${departmentId}`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}


export async function getThisMonthWorkingLogByProject(departmentId:number, st:string, projectId:string) {
    let sql = `select workinglog.dateKey, workinglog.time, workinglog.workLog, users.name from users INNER JOIN workinglog on workinglog.userId = users.userId where users.departmentId = ${departmentId} and workinglog.dateKey > "${st}" and workinglog.projectId = "${projectId}"`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

export async function getThisMonthWorkingLogByProjectAndUserName(departmentId:number, st:string, projectId:string, userName:string) {
    let sql = `select workinglog.dateKey, workinglog.time, workinglog.workLog, users.name from users INNER JOIN workinglog on workinglog.userId = users.userId where users.departmentId = ${departmentId} and workinglog.dateKey > "${st}" and workinglog.projectId = "${projectId}" and users.name = "${userName}"`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}


export async function getWorkLogListByTime(st:string, et:string) {
    let sql = `select workinglog.time, users.name, workinglog.dateKey  from users INNER JOIN workinglog on workinglog.userId = users.userId  where  workinglog.dateKey >"${st}" and workinglog.dateKey <"${et}"`;
    return await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
}

/**
 * 获取本周工时记录
 * @param userId 
 * @param startTime 
 * @param endTime 
 * @returns 
 */
export async function findWorkingLogByTimeOrUserId(departmentId:number, param, startNumber) {
    let sql = `select workinglog.workLog, users.name as name, workinglog.id as id, project.name as projectName, workinglog.time, workinglog.dateKey, `;
    sql += `workinglog.isOutside from workinglog INNER JOIN project on project.id = workinglog.projectId inner join users on users.userId = workinglog.userId  where users.departmentId=${departmentId} `;
    let paramList = Object.keys(param)
    if (paramList.length) {
        for (let i= 0; i < paramList.length; i++) {
            let key = paramList[i];
            if (key == "startTime") {
                sql += `workinglog.dateKey > '${param.startTime}' `;
            }
            if (key == "endTime") {
                sql += `workinglog.dateKey < '${param.endTime}' `;
            }
            if (key == "userId") {
                sql += `users.userId = ${param.userId}`;
            }
            if (key == "projectId") {
                sql += `workinglog.projectId = '${param.projectId}'`;
            }

            if (i != paramList.length -1) {
                sql += ' and '
            }
        }
    }
    sql += ` ORDER BY workinglog.dateKey DESC limit ${startNumber}, ${10}`;
    let res = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    return res;
}

/**
 * 获取本周工时记录数据数
 * @param userId 
 * @param startTime 
 * @param endTime 
 * @returns 
 */
export async function findWorkingLogCountByTimeOrUserId(departmentId:number, param) {
    let sql = `select count(*) as count from workinglog inner join users on users.userId = workinglog.userId where users.departmentId=${departmentId} `;
    let paramList = Object.keys(param);
    if (paramList.length) {
        for (let i= 0; i < paramList.length; i++) {
            let key = paramList[i];
            if (key == "startTime") {
                sql += `workinglog.dateKey > '${param.startTime}' `;
            }
            if (key == "endTime") {
                sql += `workinglog.dateKey < '${param.endTime}' `;
            }
            if (key == "userId") {
                sql += `users.userId = ${param.userId}`;
            }
            if (key == "projectId") {
                sql += `workinglog.projectId = '${param.projectId}'`;
            }

            if (i != paramList.length -1) {
                sql += ' and '
            }
        }
    }
    let countData = await mysqlDB.query(sql, { type: mysqlDB.QueryTypes.SELECT } );
    return countData[0].count;
}